package com.flyfish.framework.r2dbc.config.callback;

import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.domain.base.Qo;
import com.flyfish.framework.r2dbc.metadata.R2dbcMetadataManager;
import com.flyfish.framework.r2dbc.metadata.R2dbcTableMetadata;
import com.flyfish.framework.repository.DefaultReactiveRepository;
import lombok.RequiredArgsConstructor;
import org.apache.commons.collections4.CollectionUtils;
import org.reactivestreams.Publisher;
import org.springframework.beans.factory.ObjectProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.r2dbc.mapping.event.AfterConvertCallback;
import org.springframework.data.relational.core.sql.SqlIdentifier;
import org.springframework.data.util.Lazy;
import org.springframework.lang.NonNull;
import reactor.core.publisher.Mono;

import java.util.Map;
import java.util.Optional;
import java.util.function.Function;
import java.util.stream.Collectors;

@RequiredArgsConstructor
public class ReferenceR2dbcCallback implements AfterConvertCallback<Domain> {

    private final R2dbcMetadataManager r2dbcMetadataManager;
    private Lazy<Map<Class<?>, DefaultReactiveRepository<?>>> repositories;

    @Override
    @NonNull
    public Publisher<Domain> onAfterConvert(@NonNull Domain entity, @NonNull SqlIdentifier table) {
        return Mono.deferContextual(ctx -> {
            Optional<Qo<? extends Domain>> query = ctx.getOrEmpty(Qo.KEY);
            query.ifPresent(qo -> entity.setCurrentUser(qo.getUser()));
            // 判断是否需要获取关联信息
            boolean fetchRefs = query.map(Qo::isFetchRef).orElse(false);
            if (fetchRefs) {
                return doFetch(entity);
            }
            return Mono.just(entity);
        });
    }

    /**
     * 完成填充
     *
     * @param entity 实体
     * @return 结果
     */
    private Mono<Domain> doFetch(Domain entity) {
        // 遍历fields，找到要注入的数据
        R2dbcTableMetadata metadata = r2dbcMetadataManager.getMetadata(entity.getClass());
        // 尝试填充关联
        if (CollectionUtils.isNotEmpty(metadata.getAssociations())) {

        }
        return Mono.just(entity);
    }

    @Autowired
    public void setRepositories(ObjectProvider<DefaultReactiveRepository<?>> repositories) {
        this.repositories = Lazy.of(() -> repositories.stream()
                .collect(Collectors.toMap(repo -> repo.getEntityInformation().getJavaType(), Function.identity())));
    }
}
